<?php

namespace Sale\Handlers\PaySystem;

use Bitrix\Main\Localization\Loc;
use Bitrix\Main\Request;
use Bitrix\Sale\Payment;
use Bitrix\Sale\PaySystem;
use Bitrix\Main\Loader;
use Bitrix\Main\Type\DateTime;
use Bspb\Pay\Helper;
use Bspb\Pay\Pay;
use Bspb\Pay\Tables\TransactionTable;

Loc::loadMessages(__FILE__);

class bspb_payHandler extends PaySystem\ServiceHandler
{
    protected function getUrlList()
    {
        return array(
            'pay' => array(
                self::TEST_URL => 'https://mpitest.bspb.ru:5443/Exec',
                self::ACTIVE_URL => 'https://mpi.bspb.ru:5443/Exec',
            ),
        );
    }

    public function getPaymentIdFromRequest(Request $request)
    {
        return $request->get('PaymentId');
    }

    protected static function isMyResponseExtended(Request $request, $paySystemId)
    {
        return $request->get('HandlerId') == $paySystemId;
    }

    public static function getIndicativeFields()
    {
        return array('HandlerName' => 'BSPBHANDLER');
    }

    protected function isTestMode(Payment $payment = null)
    {
        return ($this->getBusinessValue(null, 'ShopIsTest') == 'Y');
    }

    public function getCurrencyList()
    {
        return array('RUB');
    }

    public function initiatePay(Payment $payment, Request $request = null)
    {
        Loader::IncludeModule('bspb.pay');
        $url = $this->getUrl($payment, 'pay');
        $params = $this->getParamsBusValue($payment);

        $pay = new Pay();
        $dataInit = $pay->Init($payment->getPaymentSystemId(), $url, $params);
        $dataInit['settings'] = array(
            'btnText' => Loc::getMessage('BPSB_BTN_PAY'),
            'newWindow' => $this->service->getField('NEW_WINDOW'),
        );

        $this->setExtraParams($dataInit);
        return $this->showTemplate($payment, 'template');
    }

    public function processRequest(Payment $payment, Request $request)
    {
        Loader::IncludeModule('bspb.pay');
        $result = new PaySystem\ServiceResult();
        $params = $this->getParamsBusValue($payment);
        $url = $this->getUrl($payment, 'pay');

        $pay = new Pay();
        $dataInit = $pay->GetOrderInformation($url, $params, $request['BankOrderId'], $request['BankSessionId']);
        if ($dataInit && $dataInit['Response']['Status'] == '00') {
            if ($dataInit['Response']['Order']['row']['Orderstatus'] == 'APPROVED') {
                $result = $this->processNoticeAction($payment, $request, $dataInit['Response']['Order']['row']);
                if ($result->isSuccess()) {
                    TransactionTable::update($request['TranID'], array('IsPaid' => true));
                    echo 'APPROVED';
                }
            } elseif ($dataInit['Response']['Order']['row']['Orderstatus'] == 'CANCELED') {
                TransactionTable::update($request['TranID'], array('IsCanceled' => true));
                echo 'CANCELED';
            } elseif ($dataInit['Response']['Order']['row']['Orderstatus'] == 'DECLINED') {
                TransactionTable::update($request['TranID'], array('IsDecline' => true));
                echo 'DECLINED';
            } elseif ($dataInit['Response']['Order']['row']['Orderstatus'] == 'EXPIRED') {
                TransactionTable::update($request['TranID'], array('IsExpired' => true));
                echo 'EXPIRED';
            }
        }

        if (!$result->isSuccess()) {
            PaySystem\ErrorLog::add(array(
                'ACTION' => 'processRequest: ',
                'MESSAGE' => join('\n', $result->getErrorMessages()),
            ));
        }

        return $result;
    }

    private function processNoticeAction(Payment $payment, Request $request, array $dataInit)
    {
        $result = new PaySystem\ServiceResult();
        $fields = array(
            'PS_STATUS' => 'Y',
            'PS_STATUS_CODE' => '00',
            'PS_STATUS_DESCRIPTION' => 'OrderID: ' . $request['BankOrderId'] . ' SessionID: ' . $request['BankSessionId'],
            'PS_STATUS_MESSAGE' => '',
            'PS_SUM' => ($dataInit['Amount'] / 100),
            'PS_CURRENCY' => 'RUB',
            'PS_RESPONSE_DATE' => new DateTime(),
            'PS_INVOICE_ID' => $dataInit['SessionID'],
        );

        if ($this->isCorrectSum($payment, $dataInit['Amount'])) {
            $data['CODE'] = 0;
            $fields['PS_STATUS'] = 'Y';
            $result->setOperationType(PaySystem\ServiceResult::MONEY_COMING);
        } else {
            $data['CODE'] = 200;
            $fields['PS_STATUS'] = 'N';
            $errorMessage = 'Incorrect payment sum';

            $result->addError(new Error($errorMessage));
            PaySystem\ErrorLog::add(array(
                'ACTION' => 'paymentAvisoResponse',
                'MESSAGE' => $errorMessage,
            ));
        }

        $result->setData($data);
        $result->setPsData($fields);

        return $result;
    }

    private function isCorrectSum(Payment $payment, $sum)
    {
        $paymentSum = $payment->getField('SUM') * 100;
        return ($this->roundByFormatCurrency($paymentSum, $payment->getField('CURRENCY'))
            == $this->roundByFormatCurrency($sum, $payment->getField('CURRENCY')));
    }

    private function roundByFormatCurrency($price, $currency)
    {
        return floatval(SaleFormatCurrency($price, $currency, false, true));
    }
}
