<?php

namespace Bspb\Pay;

use Bitrix\Main\Application;
use Bitrix\Main\Web\HttpClient;
use Bspb\Pay\Tables\Transaction;
use Bspb\Pay\Tables\TransactionTable;

class Pay
{
    public function Init($paySystemId, $url, $params)
    {
        $errors = array();
        $status = 200;

        self::checkResult();

        $infoPay = array(
            'OrderNum' => $params['OrderNum'],
            'PaymentId' => $params['PaymentId'],
            'PaymentSysId' => $paySystemId,
        );

        $urlToBank = Transaction::find($infoPay);

        if (!$urlToBank) {
            $resultUrl = $this->getResultUrl($infoPay);

            $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
            $xmlRequest = $xml->addChild('Request');
            $xmlRequest->addChild('Operation', 'CreateOrder');
            $xmlRequest->addChild('Language', $params['Language']);
            $xmlOrder = $xmlRequest->addChild('Order');
            $xmlOrder->addChild('OrderType', $params['OrderType']);
            $xmlOrder->addChild('Merchant', $params['Merchant']);
            $xmlOrder->addChild('Amount', ((float)($params['PaymentSum']) * 100));
            $xmlOrder->addChild('Currency', $params['Currency']);
            if (SITE_CHARSET != 'UTF-8') {
                $params['Description'] = mb_convert_encoding($params['Description'], 'utf-8', 'windows-1251');
            }
            $xmlOrder->addChild('Description', $params['Description']);
            $xmlOrder->addChild('ApproveURL', $resultUrl['ApproveURL']);
            $xmlOrder->addChild('CancelURL', $resultUrl['CancelURL']);
            $xmlOrder->addChild('DeclineURL', $resultUrl['DeclineURL']);
            $xmlAddParams = $xmlOrder->addChild('AddParams');
            $xmlAddParams->addChild('ShopOrderNum', $infoPay['OrderNum']);
            $xmlAddParams->addChild('ShopPaymentId', $infoPay['PaymentId']);
            $xmlAddParams->addChild('HandlerId', $infoPay['PaymentSysId']);

            $strResponse = $this->sendPostXml($url, $xml->asXML(), $params['NameCert1'], $params['NameCert2']);

            if ($params['IsLog'] == 'Y') {
                \Bitrix\Main\Diag\Debug::writeToFile("_________________________________","","bitrix/modules/bspb.pay/logs/data.log");
                \Bitrix\Main\Diag\Debug::writeToFile("CreateOrder","","bitrix/modules/bspb.pay/logs/data.log");
                \Bitrix\Main\Diag\Debug::writeToFile("_________________________________","","bitrix/modules/bspb.pay/logs/data.log");
                \Bitrix\Main\Diag\Debug::writeToFile($xml->asXML(),"","bitrix/modules/bspb.pay/logs/data.log");
                \Bitrix\Main\Diag\Debug::writeToFile("Response","","bitrix/modules/bspb.pay/logs/data.log");
                \Bitrix\Main\Diag\Debug::writeToFile($strResponse['data'],"","bitrix/modules/bspb.pay/logs/data.log");
            }


            if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
                $response = $this->XmlToArray($strResponse['data']);
                if ($response['Response']['Status'] == '00') {
                    $urlToBank = $response['Response']['Order']['URL']
                        . "?ORDERID={$response['Response']['Order']['OrderID']}&SESSIONID={$response['Response']['Order']['SessionID']}";
                    $transaction = new Transaction();
                    $transaction['OrderNum'] = $infoPay['OrderNum'];
                    $transaction['PaymentId'] = $infoPay['PaymentId'];
                    $transaction['HandlerId'] = $infoPay['PaymentSysId'];
                    $transaction['BankOrderId'] = $response['Response']['Order']['OrderID'];
                    $transaction['BankSessionId'] = $response['Response']['Order']['SessionID'];
                    $transaction['BankUrl'] = $response['Response']['Order']['URL'];
                    $transaction->save();
                } else {
                    $errors[] = 'Error: status ' . $response['Response']['Status'];
                }
            } else {
                $errors[] = 'Error: ' . $strResponse['error'];
            }
        }

        return array(
            'url' => $urlToBank,
            'errors' => $errors,
            'status' => $status,
        );
    }

    private function getResultUrl($infoPay)
    {
        $request = Application::getInstance()->getContext()->getRequest();
        $server = Application::getInstance()->getContext()->getServer();
        $url = $request->isHttps() ? 'https://' : 'http://';
        $url .= $server->getHttpHost() . '/bitrix/bspb_pages/';

        return array(
            'ApproveURL' => $url . 'approve.php',
            'CancelURL' => $url . 'cancel.php',
            'DeclineURL' => $url . 'decline.php',
        );
    }

    public function Result()
    {
        $request = Application::getInstance()->getContext()->getRequest();
        if (!$request || strlen($request['xmlmsg']) == 0) {
            return false;
        }

        $data = $this->XmlToArray($request['xmlmsg']);
        if (array_key_exists('Message', $data)) {
            $orderId = $data['Message']['OrderID'];
            $sessionId = $data['Message']['SessionID'];
        } else {
            $orderId = $data['OrderID'];
            $sessionId = $data['SessionId'];
        }

        $transaction = Transaction::findByBank(array(
            'BankOrderID' => $orderId,
            'BankSessionId' => $sessionId,
        ));
        if (!$transaction) {
            return false;
        }

        $params = array(
            'HandlerName' => 'BSPBHANDLER',
            'HandlerId' => $transaction['HandlerId'],
            'PaymentId' => $transaction['PaymentId'],
            'OrderNum' => $transaction['OrderNum'],
            'BankOrderId' => $transaction['BankOrderId'],
            'BankSessionId' => $transaction['BankSessionId'],
            'TranID' => $transaction['ID'],
        );

        $server = Application::getInstance()->getContext()->getServer();
        $httpClient = new HttpClient();
        $httpClient->setHeader('Content-Type', 'application/x-www-form-urlencoded', true);
        $url = $request->isHttps() ? 'https://' . $server->getHttpHost() : 'http://' . $server->getHttpHost();
        $url .= '/bitrix/tools/sale_ps_result.php?' . http_build_query($params);
        $params['Result'] = $httpClient->get($url);

        return $params;
    }

    public function GetOrderInformation($url, $params, $bankOrderId, $bankSessionId)
    {
        $xml = new \SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><TKKPG></TKKPG>');
        $xmlRequest = $xml->addChild('Request');
        $xmlRequest->addChild('Operation', 'GetOrderInformation');
        $xmlRequest->addChild('Language', $params['Language']);
        $xmlOrder = $xmlRequest->addChild('Order');
        $xmlOrder->addChild('Merchant', $params['Merchant']);
        $xmlOrder->addChild('OrderID', $bankOrderId);
        $xmlRequest->addChild('SessionID', $bankSessionId);
        $xmlRequest->addChild('ShowParams', 'true');
        $xmlRequest->addChild('ShowOperations', 'true');
        $xmlRequest->addChild('ClassicView', 'true');

        $strResponse = $this->sendPostXml($url, $xml->asXML(), $params['NameCert1'], $params['NameCert2']);

        if ($params['IsLog'] == 'Y') {
            \Bitrix\Main\Diag\Debug::writeToFile("_________________________________","","bitrix/modules/bspb.pay/logs/data.log");
            \Bitrix\Main\Diag\Debug::writeToFile("GetOrderInformation","","bitrix/modules/bspb.pay/logs/data.log");
            \Bitrix\Main\Diag\Debug::writeToFile("_________________________________","","bitrix/modules/bspb.pay/logs/data.log");
            \Bitrix\Main\Diag\Debug::writeToFile($xml->asXML(),"","bitrix/modules/bspb.pay/logs/data.log");
            \Bitrix\Main\Diag\Debug::writeToFile("Response","","bitrix/modules/bspb.pay/logs/data.log");
            \Bitrix\Main\Diag\Debug::writeToFile($strResponse['data'],"","bitrix/modules/bspb.pay/logs/data.log");
        }

        if (is_array($strResponse) && strlen($strResponse['error']) == 0) {
            return $this->XmlToArray($strResponse['data']);
        }

        return false;
    }

    private function sendPostXml($url, $xml, $pathCertPem, $pathCertKey)
    {
        $error = '';
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RESOLVE, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSLCERT, $_SERVER['DOCUMENT_ROOT'] . \CFile::GetPath($pathCertPem));
        curl_setopt($ch, CURLOPT_SSLKEY, $_SERVER['DOCUMENT_ROOT'] . \CFile::GetPath($pathCertKey));
        curl_setopt($ch, CURLOPT_POSTFIELDS, $xml);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_VERBOSE, 1);
        if (!$answer = curl_exec($ch)) {
            $error = 'Error with connection: ' . curl_errno($ch);
        }
        curl_close($ch);
        return array(
            'data' => $answer,
            'error' => $error
        );
    }

    private function XmlToArray($strXml)
    {
        $json = \Bitrix\Main\Web\Json::encode(simplexml_load_string($strXml));
        return \Bitrix\Main\Web\Json::decode($json);
    }

    public static function checkResult()
    {
        $listTransaction = TransactionTable::query()
            ->setFilter(array(
                'IsPaid' => false,
                'IsCanceled' => false,
                'IsDecline' => false,
                'IsRefund' => false,
                'IsExpired' => false,
            ))
            ->setSelect(array('*'))
            ->setOrder(array('ID' => 'desc'))
            ->fetchCollection();

        $server = Application::getInstance()->getContext()->getServer();
        $request = Application::getInstance()->getContext()->getRequest();
        $httpClient = new HttpClient();
        $httpClient->setHeader('Content-Type', 'application/x-www-form-urlencoded', true);
        $urlFirst = $request->isHttps() ? 'https://' . $server->getHttpHost() : 'http://' . $server->getHttpHost();
        foreach ($listTransaction as $transaction) {
            $params = array(
                'HandlerName' => 'BSPBHANDLER',
                'HandlerId' => $transaction['HandlerId'],
                'PaymentId' => $transaction['PaymentId'],
                'OrderNum' => $transaction['OrderNum'],
                'BankOrderId' => $transaction['BankOrderId'],
                'BankSessionId' => $transaction['BankSessionId'],
                'TranID' => $transaction['ID'],
            );
    
            $url = $urlFirst . '/bitrix/tools/sale_ps_result.php?' . http_build_query($params);
            $httpClient->get($url);
        }

        return '\Bspb\Pay\Pay::checkResult();';
    }
}
